#!/usr/bin/python

from sys import argv

#-----------------------------------------------------------------------

def prefer0 (a, b, excl):
    return 0

#-----------------------------------------------------------------------

def prefer1 (a, b, excl):
    za,ca = a; zb,cb = b
    if cb <= ca: return +2
    return +2 if (zb,cb-ca) in excl else -2

#-----------------------------------------------------------------------

def prefer2 (a, b, excl):
    za,ca = a; zb,cb = b
    if cb <= ca: return +2
    return +2 if (zb,cb-ca+1) in excl else -2

#-----------------------------------------------------------------------

def prefer3 (a, b, excl):
    za,ca = a; zb,cb = b
    if cb <= ca: return +2
    return -2 if (za-zb+2,ca) in excl else +2

#-----------------------------------------------------------------------

def prefer4 (a, b, excl):
    za,ca = a; zb,cb = b
    if cb <= ca: return +2
    return +2 if za*ca >= zb*cb else -2

#-----------------------------------------------------------------------

def prefer5 (a, b, excl):
    za,ca = a; zb,cb = b
    if cb <= ca: return +2
    return +2 if (za-1)*ca >= (zb-1)*cb else -2

#-----------------------------------------------------------------------

def prefer6 (a, b, excl):
    za,ca = a; zb,cb = b
    if cb <= ca: return +2
    xa = (za-zb+2,ca) in excl # excess neurons explainable
    xb = (zb,cb-ca+1) in excl # excess coins.  explainable
    if     xa and not xb: return -2
    if not xa and     xb: return +2
    if not xa and not xb: return  0
    return +1 if za*ca >= zb*cb else -1

#-----------------------------------------------------------------------

def prefer7 (a, b, excl):
    za,ca = a; zb,cb = b
    if cb <= ca: return +2
    xa = (za-zb+2,ca) in excl # excess neurons explainable
    xb = (zb,cb-ca+1) in excl # excess coins.  explainable
    if     xa and not xb: return -2
    if not xa and     xb: return +2
    if not xa and not xb: return  0
    return +1 if (za-1)*ca >= (zb-1)*cb else -1

#-----------------------------------------------------------------------

preffns = [prefer0, prefer1, prefer2, prefer3,
           prefer4, prefer5, prefer6, prefer7]

#-----------------------------------------------------------------------

if __name__ == '__main__':
    colors = ['black', '0.7white',
              'green', '0.67green', '0.35white', '0.8red', 'red']

    r    = range(1,101)         # setd efault excluded signatures
    excl = set([(i,0) for i in r] +[(0,i) for i in r]
              +[(i,1) for i in r] +[(1,i) for i in r])
    with open(argv[1], 'r') as inp:
        for line in inp:        # collect the excluded signatures
            a,b,c = line.split()
            excl.add((int(a),int(b)))

    border = set()
    for z in range(2,13):
        c = max(b for a,b in excl if a == z)
        border.add((z,c+1))
    for c in range(2,13):
        z = max(a for a,b in excl if b == c)
        border.add((z+1,c))
    border = sorted(border, key=lambda k: (-k[1],k[0]), reverse=True)

    with open('prefer.mp', 'w') as out:
        out.write('prologues    := 2;\n')
        out.write('labeloffset  := 0.5mm;\n')
        out.write('defaultscale := 0.7;\n')
        out.write('a := 3mm;\n\n')
        out.write('vardef square (expr x,y,c) =\n')
        out.write('  fill unitsquare scaled a shifted ')
        out.write('(x*a,y*a) withcolor c;\n')
        out.write('enddef;\n\n')
        out.write('vardef grid =\n')
        out.write('  pickup pencircle scaled 0.5pt;\n')
        out.write('  for x = 2 upto 13:\n')
        out.write('    draw (x*a,2a)--(x*a,13a);\n')
        out.write('    draw (2a,x*a)--(13a,x*a);\n')
        out.write('  endfor\n')
        out.write('  for x = 2 upto 12:\n')
        out.write('    b := x*a+0.5a;\n')
        out.write('    label.top(decimal(x), (15a-b,-2a)) rotated 90;\n')
        out.write('    label.top(decimal(x), (b,13a));\n')
        out.write('  endfor\n')
        out.write('  label.top("sizes", (7.5a,-1a)) rotated 90;\n')
        out.write('  label.top("coincidences", (7.5a,14a));\n')
        out.write('enddef;\n\n')
        i = 0
        for f in range(1,len(preffns)):
            prefer = preffns[f]
            k = 0
            for b in border:
                rz,rc = b
                if rz < 2 or rz > 12 or rc < 2 or rc > 12: continue
                out.write('beginfig(%d); %% ' % i)
                out.write('p_%d_%02d\n' % (f,k))
                board = dict()
                for p in excl: board[p] = 1
                board[b] = 0
                for z in range(2,rz):
                    for c in range(rc,13):
                        if (z,c) in board: continue
                        board[(z,c)] = prefer(b, (z,c), excl)+4
                for z in range(rz+1,13):
                    for c in range(2,rc+1):
                        if (z,c) in board: continue
                        board[(z,c)] = 4-prefer((z,c), b, excl)
                for p in board:
                    z,c = p
                    if z < 2 or z > 12 or c < 2 or c > 12: continue
                    x = board[p]
                    out.write('  square(%d,%d,%s);\n'
                              % (c,14-z,colors[x]))
                out.write('  grid;\n')
                out.write('endfig;\n\n')
                i += 1; k += 1
        out.write('end.\n')
